/*
 **************************************************************************
 * Copyright (c) 2016-2017 The Linux Foundation. All rights reserved.
 *
 * Permission to use, copy, modify, and/or distribute this software for
 * any purpose with or without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
 * OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 **************************************************************************
 */

#include <common.h>
#include "ipq807x_ppe.h"

#define pr_info(fmt, args...) printf(fmt, ##args);

/*
 * ipq807x_ppe_gpio_reg_write()
 */
static inline void ipq807x_ppe_gpio_reg_write(u32 reg, u32 val)
{
	writel(val, IPQ807X_PPE_FPGA_GPIO_BASE_ADDR + reg);
}

/*
 * ipq807x_ppe_reg_read()
 */
static inline void ipq807x_ppe_reg_read(u32 reg, u32 *val)
{
	*val = readl((void *)(IPQ807X_PPE_BASE_ADDR + reg));
}

/*
 * ipq807x_ppe_reg_write()
 */
static inline void ipq807x_ppe_reg_write(u32 reg, u32 val)
{
	writel(val, (void *)(IPQ807X_PPE_BASE_ADDR + reg));
}

/*
 * ipq807x_ppe_vp_port_tbl_set()
 */
static void ipq807x_ppe_vp_port_tbl_set(int port, int vsi)
{
	u32 addr = IPQ807X_PPE_L3_VP_PORT_TBL_ADDR +
		 (port * IPQ807X_PPE_L3_VP_PORT_TBL_INC);
	ipq807x_ppe_reg_write(addr, 0x0);
	ipq807x_ppe_reg_write(addr + 0x4 , 1 << 9 | vsi << 10);
	ipq807x_ppe_reg_write(addr + 0x8, 0x0);
}

/*
 * ipq807x_ppe_ucast_queue_map_tbl_queue_id_set()
 */
static void ipq807x_ppe_ucast_queue_map_tbl_queue_id_set(int queue, int port)
{
	uint32_t val;

	ipq807x_ppe_reg_read(IPQ807X_PPE_QM_UQM_TBL +
		 (port * IPQ807X_PPE_UCAST_QUEUE_MAP_TBL_INC), &val);

	val |= queue << 4;

	ipq807x_ppe_reg_write(IPQ807X_PPE_QM_UQM_TBL +
		 (port * IPQ807X_PPE_UCAST_QUEUE_MAP_TBL_INC), val);
}

/*
 * ipq807x_vsi_setup()
 */
static void ipq807x_vsi_setup(int vsi, uint8_t group_mask)
{
	uint32_t val = (group_mask << 24 | group_mask << 16 | group_mask << 8
							    | group_mask);

	/* Set mask */
	ipq807x_ppe_reg_write(0x061800 + (vsi * 0x10), val);

	/*  new addr lrn en | station move lrn en */
	ipq807x_ppe_reg_write(0x061804 + (vsi * 0x10), 0x9);
}

/*
 * ipq807x_gmac_enable()
 */

static void ipq807x_gmac_enable(void)
{
	writel(0x0, 0x1008004);
}

/*
 * ipq807x_gmac_port_enable()
 */
static void ipq807x_gmac_port_enable(int port)
{
	ipq807x_ppe_reg_write(IPQ807X_PPE_MAC_ENABLE + (0x200 * port), 0x13);
	ipq807x_ppe_reg_write(IPQ807X_PPE_MAC_SPEED + (0x200 * port), 0x2);
	ipq807x_ppe_reg_write(IPQ807X_PPE_MAC_MIB_CTL + (0x200 * port), 0x1);
}

/*
 * ipq807x_ppe_flow_port_map_tbl_port_num_set()
 */
static void ipq807x_ppe_flow_port_map_tbl_port_num_set(int queue, int port)
{
	ipq807x_ppe_reg_write(IPQ807X_PPE_L0_FLOW_PORT_MAP_TBL +
			queue * IPQ807X_PPE_L0_FLOW_PORT_MAP_TBL_INC, port);
	ipq807x_ppe_reg_write(IPQ807X_PPE_L1_FLOW_PORT_MAP_TBL +
			port * IPQ807X_PPE_L1_FLOW_PORT_MAP_TBL_INC, port);
}

/*
 * ipq807x_ppe_flow_map_tbl_set()
 */
static void ipq807x_ppe_flow_map_tbl_set(int queue, int port)
{
	uint32_t val = port | 0x401000; /* c_drr_wt = 1, e_drr_wt = 1 */
	ipq807x_ppe_reg_write(IPQ807X_PPE_L0_FLOW_MAP_TBL + queue * IPQ807X_PPE_L0_FLOW_MAP_TBL_INC,
									val);

	val = port | 0x100400; /* c_drr_wt = 1, e_drr_wt = 1 */
	ipq807x_ppe_reg_write(IPQ807X_PPE_L1_FLOW_MAP_TBL + port * IPQ807X_PPE_L1_FLOW_MAP_TBL_INC,
									val);
}

/*
 * ipq807x_ppe_tdm_configuration
 */
static void ipq807x_ppe_tdm_configuration(void)
{
	int i = 0;

	/*
	 * TDM is configured with instructions for each tick
	 * Port/action are configured as given below
	 *
	 * 0x5:0x5	TDM_CFG_VALID		0:idle tick
	 * 0x4:0x4	TDM_CFG_DIR		0:ingress wr
	 *					1:egress rd
	 * 0x3:0x0	TDM_CFG_PORT_NUM	0:DMA
	 *					1~4:Ethernet 1G
	 *					5~6:Ethernet 5G
	 *					7~8:Security0/1
	 */
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_XGMAC1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_XGMAC1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_XGMAC2);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_XGMAC2);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_QCOM1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_CRYPTO1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_CRYPTO1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_XGMAC1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_XGMAC1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_XGMAC2);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_XGMAC2);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_QCOM3);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_CRYPTO1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_CRYPTO1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_XGMAC1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_XGMAC1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_XGMAC2);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_XGMAC2);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_CRYPTO1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_CRYPTO1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_XGMAC1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_XGMAC1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_XGMAC2);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_XGMAC2);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_QCOM2);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_CRYPTO1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_CRYPTO1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_XGMAC1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_XGMAC1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_QCOM4);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_XGMAC2);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_XGMAC2);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_CRYPTO1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_CRYPTO1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_XGMAC1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_XGMAC1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID
			| IPQ807X_PPE_TDM_CFG_DIR_EGRESS
			| IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_XGMAC2);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_XGMAC2);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_QCOM1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_CRYPTO1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_XGMAC1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_XGMAC1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_XGMAC2);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_XGMAC2);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_QCOM3);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_CRYPTO1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_CRYPTO1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_XGMAC1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_XGMAC1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_XGMAC2);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_XGMAC2);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_CRYPTO1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_CRYPTO1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_XGMAC1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_XGMAC1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_XGMAC2);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_XGMAC2);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_QCOM2);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_CRYPTO1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_CRYPTO1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_XGMAC1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_XGMAC1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_QCOM4);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_XGMAC2);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_XGMAC2);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_INGRESS |
			IPQ807X_PPE_PORT_CRYPTO1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_GPIO_OFFSET + (i++ * 0x10),
			IPQ807X_PPE_TDM_CFG_VALID |
			IPQ807X_PPE_TDM_CFG_DIR_EGRESS |
			IPQ807X_PPE_PORT_CRYPTO1);

	/* Set TDM Depth to 100 entries */
	ipq807x_ppe_reg_write(IPQ807X_PPE_TDM_CFG_DEPTH_OFFSET, IPQ807X_PPE_TDM_CFG_DEPTH_VAL);
}

/*
 * ipq807x_ppe_sched_configuration
 */
static void ipq807x_ppe_sched_configuration(void)
{
	int i = 0;

	/*
	 * PSCH_TDM_CFG_TBL_DES_PORT : determine which egress port traffic
	 *			will be selected and transmitted out
	 * PSCH_TDM_CFG_TBL_ENS_PORT : determine which portâ€™s queue need
	 *			to be linked to scheduler at the current tick
	 * PSCH_TDM_CFG_TBL_ENS_PORT_BITMAP : determine port bitmap
	 *			for source of queue
	 *
	 * 0xf:0x8	PSCH_TDM_CFG_TBL_ENS_PORT_BITMAP	1110_1110
	 *							(Port:765-432)
	 *
	 * 0x7:0x4	PSCH_TDM_CFG_TBL_ENS_PORT		0:DMA
	 *							1~4:Ethernet 1G
	 *							5~6:Ethernet 5G
	 *							7~8:Security0/1
	 *
	 * 0x3:0x0	PSCH_TDM_CFG_TBL_DES_PORT		0:DMA
	 *							1~4:Ethernet 1G
	 *							5~6:Ethernet 5G
	 *							7~8:Security0/1
	 *
	 * For eg, 0xee60 =((IPQ807X_PPE_PORT_CRYPTO1_BITPOS | IPQ807X_PPE_PORT_XGMAC2_BITPOS |
	 *		IPQ807X_PPE_PORT_XGMAC1_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
	 *		IPQ807X_PPE_PORT_QCOM2_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
	 *		IPQ807X_PPE_PORT_XGMAC2 | IPQ807X_PPE_PORT_EDMA);
	 */
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_CRYPTO1_BITPOS | IPQ807X_PPE_PORT_XGMAC2_BITPOS |
			IPQ807X_PPE_PORT_XGMAC1_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
			IPQ807X_PPE_PORT_QCOM2_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_XGMAC2 << 4) | IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_CRYPTO1_BITPOS | IPQ807X_PPE_PORT_XGMAC2_BITPOS |
			IPQ807X_PPE_PORT_QCOM3_BITPOS | IPQ807X_PPE_PORT_QCOM2_BITPOS |
			IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_QCOM4 << 4) | IPQ807X_PPE_PORT_XGMAC1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_CRYPTO1_BITPOS | IPQ807X_PPE_PORT_QCOM4_BITPOS |
			IPQ807X_PPE_PORT_QCOM3_BITPOS | IPQ807X_PPE_PORT_QCOM2_BITPOS |
			IPQ807X_PPE_PORT_QCOM1_BITPOS | IPQ807X_PPE_PORT_EDMA_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_EDMA << 4) | IPQ807X_PPE_PORT_XGMAC2);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_CRYPTO1_BITPOS | IPQ807X_PPE_PORT_XGMAC1_BITPOS |
			IPQ807X_PPE_PORT_QCOM4_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
			IPQ807X_PPE_PORT_QCOM2_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_XGMAC1 << 4) | IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_XGMAC2_BITPOS | IPQ807X_PPE_PORT_XGMAC1_BITPOS |
			IPQ807X_PPE_PORT_QCOM4_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
			IPQ807X_PPE_PORT_QCOM2_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_XGMAC2 << 4) | IPQ807X_PPE_PORT_CRYPTO1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_EDMA_BITPOS | IPQ807X_PPE_PORT_XGMAC2_BITPOS |
			IPQ807X_PPE_PORT_QCOM4_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
			IPQ807X_PPE_PORT_QCOM2_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_EDMA << 4) | IPQ807X_PPE_PORT_XGMAC1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_CRYPTO1_BITPOS | IPQ807X_PPE_PORT_EDMA_BITPOS |
			IPQ807X_PPE_PORT_QCOM4_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
			IPQ807X_PPE_PORT_QCOM2_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_CRYPTO1 << 4) | IPQ807X_PPE_PORT_XGMAC2);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_CRYPTO1_BITPOS | IPQ807X_PPE_PORT_XGMAC1_BITPOS |
			IPQ807X_PPE_PORT_QCOM4_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
			IPQ807X_PPE_PORT_QCOM2_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_XGMAC1 << 4) | IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_CRYPTO1_BITPOS | IPQ807X_PPE_PORT_XGMAC1_BITPOS |
			IPQ807X_PPE_PORT_QCOM4_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
			IPQ807X_PPE_PORT_QCOM2_BITPOS | IPQ807X_PPE_PORT_XGMAC2_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_XGMAC2 << 4) | IPQ807X_PPE_PORT_QCOM1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_CRYPTO1_BITPOS | IPQ807X_PPE_PORT_XGMAC2_BITPOS |
			IPQ807X_PPE_PORT_QCOM4_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
			IPQ807X_PPE_PORT_QCOM2_BITPOS | IPQ807X_PPE_PORT_EDMA_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_EDMA << 4) | IPQ807X_PPE_PORT_XGMAC1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_CRYPTO1_BITPOS | IPQ807X_PPE_PORT_XGMAC2_BITPOS |
			IPQ807X_PPE_PORT_QCOM4_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
			IPQ807X_PPE_PORT_QCOM2_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_QCOM1 << 4) | IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_CRYPTO1_BITPOS | IPQ807X_PPE_PORT_XGMAC1_BITPOS |
			IPQ807X_PPE_PORT_QCOM4_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
			IPQ807X_PPE_PORT_QCOM2_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_XGMAC1 << 4) | IPQ807X_PPE_PORT_XGMAC2);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_CRYPTO1_BITPOS | IPQ807X_PPE_PORT_XGMAC1_BITPOS |
			IPQ807X_PPE_PORT_QCOM4_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
			IPQ807X_PPE_PORT_EDMA_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_EDMA  << 4) | IPQ807X_PPE_PORT_QCOM2);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_CRYPTO1_BITPOS | IPQ807X_PPE_PORT_XGMAC2_BITPOS |
			IPQ807X_PPE_PORT_QCOM4_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
			IPQ807X_PPE_PORT_EDMA_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_XGMAC2 << 4) | IPQ807X_PPE_PORT_XGMAC1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_CRYPTO1_BITPOS | IPQ807X_PPE_PORT_XGMAC2_BITPOS |
			IPQ807X_PPE_PORT_QCOM4_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
			IPQ807X_PPE_PORT_QCOM2_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_QCOM2 << 4) | IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_CRYPTO1_BITPOS | IPQ807X_PPE_PORT_XGMAC1_BITPOS |
			IPQ807X_PPE_PORT_QCOM4_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
			IPQ807X_PPE_PORT_QCOM2_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_XGMAC1 << 4) | IPQ807X_PPE_PORT_XGMAC2);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_XGMAC1_BITPOS | IPQ807X_PPE_PORT_EDMA_BITPOS |
			IPQ807X_PPE_PORT_QCOM4_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
			IPQ807X_PPE_PORT_QCOM2_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_EDMA << 4) | IPQ807X_PPE_PORT_CRYPTO1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_XGMAC2_BITPOS | IPQ807X_PPE_PORT_XGMAC1_BITPOS |
			IPQ807X_PPE_PORT_QCOM4_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
			IPQ807X_PPE_PORT_QCOM2_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_XGMAC2 << 4) | IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_CRYPTO1_BITPOS | IPQ807X_PPE_PORT_XGMAC2_BITPOS |
			IPQ807X_PPE_PORT_QCOM4_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
			IPQ807X_PPE_PORT_QCOM2_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_CRYPTO1 << 4) | IPQ807X_PPE_PORT_XGMAC1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_CRYPTO1_BITPOS | IPQ807X_PPE_PORT_EDMA_BITPOS |
			IPQ807X_PPE_PORT_QCOM4_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
			IPQ807X_PPE_PORT_QCOM2_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_EDMA << 4) | IPQ807X_PPE_PORT_XGMAC2);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_CRYPTO1_BITPOS | IPQ807X_PPE_PORT_XGMAC1_BITPOS |
			IPQ807X_PPE_PORT_QCOM4_BITPOS | IPQ807X_PPE_PORT_EDMA_BITPOS |
			IPQ807X_PPE_PORT_QCOM2_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_XGMAC1 << 4) | IPQ807X_PPE_PORT_QCOM3);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_CRYPTO1_BITPOS | IPQ807X_PPE_PORT_XGMAC2_BITPOS |
			IPQ807X_PPE_PORT_QCOM4_BITPOS | IPQ807X_PPE_PORT_XGMAC1_BITPOS |
			IPQ807X_PPE_PORT_QCOM2_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_XGMAC2 << 4) | IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_CRYPTO1_BITPOS | IPQ807X_PPE_PORT_XGMAC2_BITPOS |
			IPQ807X_PPE_PORT_QCOM4_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
			IPQ807X_PPE_PORT_QCOM2_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_QCOM3 << 4) | IPQ807X_PPE_PORT_XGMAC1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_CRYPTO1_BITPOS | IPQ807X_PPE_PORT_EDMA_BITPOS |
			IPQ807X_PPE_PORT_QCOM4_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
			IPQ807X_PPE_PORT_QCOM2_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_EDMA << 4) | IPQ807X_PPE_PORT_XGMAC2);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_CRYPTO1_BITPOS | IPQ807X_PPE_PORT_XGMAC1_BITPOS |
			IPQ807X_PPE_PORT_QCOM4_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
			IPQ807X_PPE_PORT_QCOM2_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_XGMAC1 << 4) | IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_CRYPTO1_BITPOS | IPQ807X_PPE_PORT_XGMAC2_BITPOS |
			IPQ807X_PPE_PORT_XGMAC1_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
			IPQ807X_PPE_PORT_QCOM2_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_XGMAC2 << 4) | IPQ807X_PPE_PORT_QCOM4);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_CRYPTO1_BITPOS | IPQ807X_PPE_PORT_EDMA_BITPOS |
			IPQ807X_PPE_PORT_XGMAC2_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
			IPQ807X_PPE_PORT_QCOM2_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_EDMA << 4) | IPQ807X_PPE_PORT_XGMAC1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_CRYPTO1_BITPOS | IPQ807X_PPE_PORT_EDMA_BITPOS |
			IPQ807X_PPE_PORT_QCOM4_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
			IPQ807X_PPE_PORT_QCOM2_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_QCOM4 << 4) | IPQ807X_PPE_PORT_XGMAC2);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_CRYPTO1_BITPOS | IPQ807X_PPE_PORT_XGMAC1_BITPOS |
			IPQ807X_PPE_PORT_QCOM4_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
			IPQ807X_PPE_PORT_QCOM2_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_XGMAC1 << 4) | IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_XGMAC2_BITPOS | IPQ807X_PPE_PORT_XGMAC1_BITPOS |
			IPQ807X_PPE_PORT_QCOM4_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
			IPQ807X_PPE_PORT_QCOM2_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_XGMAC2 << 4) | IPQ807X_PPE_PORT_CRYPTO1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_EDMA_BITPOS | IPQ807X_PPE_PORT_XGMAC2_BITPOS |
			IPQ807X_PPE_PORT_QCOM4_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
			IPQ807X_PPE_PORT_QCOM2_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_EDMA << 4) | IPQ807X_PPE_PORT_XGMAC1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_CRYPTO1_BITPOS | IPQ807X_PPE_PORT_XGMAC2_BITPOS |
			IPQ807X_PPE_PORT_QCOM4_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
			IPQ807X_PPE_PORT_QCOM2_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_CRYPTO1 << 4) | IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_CRYPTO1_BITPOS | IPQ807X_PPE_PORT_XGMAC1_BITPOS |
			IPQ807X_PPE_PORT_QCOM4_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
			IPQ807X_PPE_PORT_QCOM2_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_XGMAC1 << 4) | IPQ807X_PPE_PORT_XGMAC2);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_CRYPTO1_BITPOS | IPQ807X_PPE_PORT_XGMAC1_BITPOS |
			IPQ807X_PPE_PORT_QCOM4_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
			IPQ807X_PPE_PORT_EDMA_BITPOS | IPQ807X_PPE_PORT_QCOM2_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_EDMA << 4) | IPQ807X_PPE_PORT_QCOM1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_CRYPTO1_BITPOS | IPQ807X_PPE_PORT_XGMAC2_BITPOS |
			IPQ807X_PPE_PORT_QCOM4_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
			IPQ807X_PPE_PORT_QCOM2_BITPOS | IPQ807X_PPE_PORT_EDMA_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_XGMAC2 << 4) | IPQ807X_PPE_PORT_XGMAC1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_CRYPTO1_BITPOS | IPQ807X_PPE_PORT_XGMAC2_BITPOS |
			IPQ807X_PPE_PORT_QCOM4_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
			IPQ807X_PPE_PORT_QCOM2_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_QCOM1 << 4) | IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_CRYPTO1_BITPOS | IPQ807X_PPE_PORT_XGMAC1_BITPOS |
			IPQ807X_PPE_PORT_QCOM4_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
			IPQ807X_PPE_PORT_QCOM2_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_XGMAC1 << 4) | IPQ807X_PPE_PORT_XGMAC2);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_CRYPTO1_BITPOS | IPQ807X_PPE_PORT_XGMAC1_BITPOS |
			IPQ807X_PPE_PORT_QCOM4_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
			IPQ807X_PPE_PORT_EDMA_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_EDMA << 4) | IPQ807X_PPE_PORT_QCOM2);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_CRYPTO1_BITPOS | IPQ807X_PPE_PORT_XGMAC1_BITPOS |
			IPQ807X_PPE_PORT_XGMAC2_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
			IPQ807X_PPE_PORT_QCOM4_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_XGMAC2 << 4) | IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_CRYPTO1_BITPOS | IPQ807X_PPE_PORT_XGMAC2_BITPOS |
			IPQ807X_PPE_PORT_QCOM4_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
			IPQ807X_PPE_PORT_QCOM2_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_QCOM2 << 4) | IPQ807X_PPE_PORT_XGMAC1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_CRYPTO1_BITPOS | IPQ807X_PPE_PORT_EDMA_BITPOS |
			IPQ807X_PPE_PORT_QCOM4_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
			IPQ807X_PPE_PORT_QCOM2_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_EDMA << 4) | IPQ807X_PPE_PORT_XGMAC2);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_XGMAC1_BITPOS | IPQ807X_PPE_PORT_EDMA_BITPOS |
			IPQ807X_PPE_PORT_QCOM4_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
			IPQ807X_PPE_PORT_QCOM2_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_XGMAC1 << 4) | IPQ807X_PPE_PORT_CRYPTO1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_XGMAC2_BITPOS | IPQ807X_PPE_PORT_XGMAC1_BITPOS |
			IPQ807X_PPE_PORT_QCOM4_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
			IPQ807X_PPE_PORT_QCOM2_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_XGMAC2 << 4) | IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_CRYPTO1_BITPOS | IPQ807X_PPE_PORT_XGMAC2_BITPOS |
			IPQ807X_PPE_PORT_QCOM4_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
			IPQ807X_PPE_PORT_QCOM2_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_CRYPTO1 << 4) | IPQ807X_PPE_PORT_XGMAC1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_CRYPTO1_BITPOS | IPQ807X_PPE_PORT_EDMA_BITPOS |
			IPQ807X_PPE_PORT_QCOM4_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
			IPQ807X_PPE_PORT_QCOM2_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_EDMA << 4) | IPQ807X_PPE_PORT_XGMAC2);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_CRYPTO1_BITPOS | IPQ807X_PPE_PORT_XGMAC1_BITPOS |
			IPQ807X_PPE_PORT_QCOM4_BITPOS | IPQ807X_PPE_PORT_EDMA_BITPOS |
			IPQ807X_PPE_PORT_QCOM2_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_XGMAC1 << 4) | IPQ807X_PPE_PORT_QCOM3);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_CRYPTO1_BITPOS | IPQ807X_PPE_PORT_XGMAC2_BITPOS |
			IPQ807X_PPE_PORT_QCOM4_BITPOS | IPQ807X_PPE_PORT_XGMAC1_BITPOS |
			IPQ807X_PPE_PORT_QCOM2_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_XGMAC2 << 4) | IPQ807X_PPE_PORT_EDMA);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_CRYPTO1_BITPOS | IPQ807X_PPE_PORT_XGMAC2_BITPOS |
			IPQ807X_PPE_PORT_QCOM4_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
			IPQ807X_PPE_PORT_QCOM2_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_QCOM3 << 4) | IPQ807X_PPE_PORT_XGMAC1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_CRYPTO1_BITPOS | IPQ807X_PPE_PORT_EDMA_BITPOS |
			IPQ807X_PPE_PORT_QCOM4_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
			IPQ807X_PPE_PORT_QCOM2_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_EDMA << 4) | IPQ807X_PPE_PORT_XGMAC2);
	ipq807x_ppe_reg_write(IPQ807X_PPE_FPGA_SCHED_OFFSET + (i++ * 0x10),
			((IPQ807X_PPE_PORT_CRYPTO1_BITPOS | IPQ807X_PPE_PORT_XGMAC1_BITPOS |
			IPQ807X_PPE_PORT_EDMA_BITPOS | IPQ807X_PPE_PORT_QCOM3_BITPOS |
			IPQ807X_PPE_PORT_QCOM2_BITPOS | IPQ807X_PPE_PORT_QCOM1_BITPOS) << 8) |
			(IPQ807X_PPE_PORT_XGMAC1 << 4) | IPQ807X_PPE_PORT_QCOM4);

	/* Set Sched Depth to 50 entries */
	ipq807x_ppe_reg_write(IPQ807X_PPE_TDM_SCHED_DEPTH_OFFSET, IPQ807X_PPE_TDM_SCHED_DEPTH_VAL);
}

/*
 * ipq807x_ppe_c_sp_cfg_tbl_drr_id_set
 */
static void ipq807x_ppe_c_sp_cfg_tbl_drr_id_set(int id)
{
	ipq807x_ppe_reg_write(IPQ807X_PPE_L0_C_SP_CFG_TBL + (id * 0x80), id * 2);
	ipq807x_ppe_reg_write(IPQ807X_PPE_L1_C_SP_CFG_TBL + (id * 0x80), id * 2);
}

/*
 * ipq807x_ppe_e_sp_cfg_tbl_drr_id_set
 */
static void ipq807x_ppe_e_sp_cfg_tbl_drr_id_set(int id)
{
	ipq807x_ppe_reg_write(IPQ807X_PPE_L0_E_SP_CFG_TBL + (id * 0x80), id * 2 + 1);
	ipq807x_ppe_reg_write(IPQ807X_PPE_L1_E_SP_CFG_TBL + (id * 0x80), id * 2 + 1);
}

/*
 * ipq807x_ppe_provision_init()
 */
void ipq807x_ppe_provision_init(void)
{
	int i;
	uint32_t queue;

	/* Port4 Port5, Port6 port mux configuration, all GMAC */
	writel(0x3d, 0x3a000010);

	/* tdm/sched configuration */
	ipq807x_ppe_tdm_configuration();
	ipq807x_ppe_sched_configuration();

	ipq807x_gmac_enable();

	/* disable clock gating */
	ipq807x_ppe_reg_write(0x000008, 0x0);

	/* flow ctrl disable */
	ipq807x_ppe_reg_write(0x200368, 0xc88);

#ifdef CONFIG_IPQ807X_BRIDGED_MODE
	/* Add CPU port 0 to VSI 2 */
	ipq807x_ppe_vp_port_tbl_set(0, 2);

	/* Add port 1 - 4 to VSI 2 */
	ipq807x_ppe_vp_port_tbl_set(1, 2);
	ipq807x_ppe_vp_port_tbl_set(2, 2);
	ipq807x_ppe_vp_port_tbl_set(3, 2);
	ipq807x_ppe_vp_port_tbl_set(4, 2);

#else
	ipq807x_ppe_vp_port_tbl_set(1, 2);
	ipq807x_ppe_vp_port_tbl_set(2, 3);
	ipq807x_ppe_vp_port_tbl_set(3, 4);
	ipq807x_ppe_vp_port_tbl_set(4, 5);
#endif

	/* Unicast priority map */
	ipq807x_ppe_reg_write(IPQ807X_PPE_QM_UPM_TBL, 0);

	/* Port0 - 7 unicast queue settings */
	for (i = 0; i < 8; i++) {
		if (i == 0)
			queue = 0;
		else
			queue = ((i * 0x10) + 0x70);

		ipq807x_ppe_ucast_queue_map_tbl_queue_id_set(queue, i);
		ipq807x_ppe_flow_port_map_tbl_port_num_set(queue, i);
		ipq807x_ppe_flow_map_tbl_set(queue, i);
		ipq807x_ppe_c_sp_cfg_tbl_drr_id_set(i);
		ipq807x_ppe_e_sp_cfg_tbl_drr_id_set(i);
	}

	/* Port0 multicast queue */
	ipq807x_ppe_reg_write(0x409000, 0x00000000);
	ipq807x_ppe_reg_write(0x403000, 0x00401000);

	/* Port1 - 7 multicast queue */
	for (i = 1; i < 8; i++) {
		ipq807x_ppe_reg_write(0x409100 + ((i - 1) * 0x40), i);
		ipq807x_ppe_reg_write(0x403100 + ((i - 1) * 0x40), 0x401000 | i);
	}

	/*
	 * Port0 - Port7 learn enable and isolation port bitmap and TX_EN
	 * Here please pay attention on bit16 (TX_EN) is not set on port7
	 */
	for (i = 0; i < 7; i++)
		ipq807x_ppe_reg_write(IPQ807X_PPE_PORT_BRIDGE_CTRL_OFFSET + (i * 4),
			      IPQ807X_PPE_PORT_BRIDGE_CTRL_PROMISC_EN |
			      IPQ807X_PPE_PORT_BRIDGE_CTRL_TXMAC_EN |
			      IPQ807X_PPE_PORT_BRIDGE_CTRL_PORT_ISOLATION_BMP |
			      IPQ807X_PPE_PORT_BRIDGE_CTRL_STATION_LRN_EN |
			      IPQ807X_PPE_PORT_BRIDGE_CTRL_NEW_ADDR_LRN_EN);

	ipq807x_ppe_reg_write(IPQ807X_PPE_PORT_BRIDGE_CTRL_OFFSET + (7 * 4),
		      IPQ807X_PPE_PORT_BRIDGE_CTRL_PROMISC_EN |
		      IPQ807X_PPE_PORT_BRIDGE_CTRL_PORT_ISOLATION_BMP |
		      IPQ807X_PPE_PORT_BRIDGE_CTRL_STATION_LRN_EN |
		      IPQ807X_PPE_PORT_BRIDGE_CTRL_NEW_ADDR_LRN_EN);

	/* Global learning */
	ipq807x_ppe_reg_write(0x060038, 0xc0);

#ifdef CONFIG_IPQ807X_BRIDGED_MODE
	ipq807x_vsi_setup(2, 0x1f);
#else
	ipq807x_vsi_setup(2, 0x03);
	ipq807x_vsi_setup(3, 0x05);
	ipq807x_vsi_setup(4, 0x09);
	ipq807x_vsi_setup(5, 0x11);
#endif

	/* Port 0-7 STP */
	for (i = 0; i < 8; i++)
		ipq807x_ppe_reg_write(IPQ807X_PPE_STP_BASE + (0x4 * i), 0x3);

	/* Port 0-5 enable */
	for (i = 0; i < 6; i++)
		ipq807x_gmac_port_enable(i);
}
